/* Mini centrale d'alarme
   Cette application va vous permettre de réaliser une petite centrale d'alarme anti-intrusion
   dotée de 2 zones et d'un affichage sur écran LCD. 
  
   CE PROGRAMME EST FOURNI «TEL QUEL», SANS GARANTIE D'AUCUNE SORTE, EXPLICITE OU
   IMPLICITE, Y COMPRIS MAIS SANS S'Y LIMITER LES GARANTIES DE QUALITÉ MARCHANDE,
   D'ADAPTATION À UN USAGE PARTICULIER ET D'ABSENCE DE CONTREFAÇON. EN AUCUN CAS, LA
   RESPONSABILITE DES AUTEURS OU DES DÉTENTEURS DE DROITS D'AUTEUR NE SERONT TENUS 
   RESPONSABLES DE TOUTE RECLAMATION, DOMMAGE OU AUTRE RESPONSABILITÉ, QUE CE SOIT
   DANS UNE ACTION DE CONTRAT, TORT OU AUTREMENT, PROVENANT DE, OU EN RELATION AVEC
   CE PROGRAMME OU L'UTILISATION OU AUTRES TRAITEMENTS DU PROGRAMME. CE PROGRAMME 
   EST PUBLIE SOUS LICENCE GNU GPL.
     
*/

#include <LiquidCrystal.h>                // Ajout de la librairie permettant de gérer l'afficheur LCD
LiquidCrystal lcd(11, 12, 7, 8, 9, 10);   // Attribution des broches de la platine destinées à piloter l'afficheur
const byte Clef = 2;                      // Attribution de la Clef "M/A" sur le port 2 de la platine "Flip & Click"
const byte ZoneInst = 3;                  // Attribution de la zone instatanée au port 3 de la platine "Flip & Click"
const byte ZoneRet = 5;                   // Attribution de la zone retardée au port 5 de la platine "Flip & Click"
const byte Buzzer = 6;                    // Attribution du buzzer au port 6 de la platine "Flip & Click"
byte Tempo;                               // Variable générale pour la gestion des temporisations
byte MemZoneI = 0;                        // Memoire alarme zone instantanée
byte MemZoneR = 0;                        // Memoire alarme zone Retardée
int Compteur;                             // Variable mémoire pour génération des bips sonores fugitifs
const byte TempoSortie = 15;              // Durée de la temporisation de sortie
const byte TempoEntree = 10;              // Durée de la temporisation d'entrée
const byte TempoAlarme = 180;             // Durée de la temporisation d'alarme
const char NomZoneI[] = "Salon";          // Nom de la zone instantannée
const char NomZoneR[] = "Couloir";        // Nom de la zone retardée

void setup() 
{
  lcd.begin(16, 2);                       // Déclaration d'un afficheur LCD doté de 2 lignes de 16 caractères
  pinMode(Clef, INPUT);                   // Le port sur lequel est connecté la Clef est une entrée
  pinMode(ZoneInst, INPUT);               // Le port sur lequel est connecté la zone instantanée est une entrée
  pinMode(ZoneRet, INPUT);                // Le port sur lequel est connecté la zone retardée est une entrée
  Arret ();                               // Afffiche le message "centrale à l'arrêt"
}

void loop() {
  Compteur = 0;                           // Initialise compteur pour génération de bips sonores fugitifs en cas de mémoire alarme
  while (digitalRead(Clef) == HIGH)       // Tant que l'on a pas mis la centrale en "marche"
  {
  if (MemZoneI == 1 | MemZoneR ==1)       // Test s'il y a une mémoire alarme ? 
  {
    if (Compteur > 2000000)
      {
        analogWrite(Buzzer, 80);          // Bip sonore fugitif
        delay(10);
        analogWrite(Buzzer, 0);
        Compteur = 0;                     // Réinitialisation compteur bip sonores fugitifs
      }
  } 
   Compteur ++;                           // Incrémente le compteur pour la génération de Bip sonores fugitifs en cas d'alarme                               
 }
  delay(50);                              // Temporisation pour l'intégration du rebond de la clef "M/A"
  lcd.clear();                            // Efface l'écran et place le curseur en haut à gauche 
  lcd.print("Temporisation de");          // Affichage de l'activation de la  temporisation de sortie
  lcd.setCursor(0, 1);                    // Place le curseur sur la 2ème ligne
  lcd.print("sortie active   ");
  MemZoneI = 0;                           // Initialise mémoire alarme de la zone instantanée
  MemZoneR = 0;                           // Initialise mémoire alarme de la zone retardée
  Tempo = TempoSortie;                    // Récupère la valeur de la temporisation de sortie

  while (digitalRead(Clef) == LOW  &  Tempo !=255)     // Tant que la teporisation de sortie n'est pas fini ET que l'on a pas arrété la centrale d'alarme
  {  
    lcd.setCursor(14, 1);                 // Place le curseur sur la 2ème ligne pour afficher la temporiation
    lcd.print(Tempo);                     // Affiche la valeur de la temporisation de sortie
    lcd.print(" ");                       // Affiche un caractère à "vide" pour effacer les "anciens" digits 
    analogWrite(Buzzer, 80);              // Bip sonore
    delay(80);
    analogWrite(Buzzer, 0);           
    Tempo--;                              // Décompte la durée temporisation de sortie
    delay(920);
  }
  
   if (digitalRead(Clef) == LOW) Actif (); // Annonce que la centrale est armée si la clef est toujours sur la position "Marche"


//---------------------------------------------------- La centrale est armée et surveille ses 2 zones ------

   while (digitalRead(Clef) == LOW)        // Tant que l'on a pas mis la centrale d'alarme à l'arrêt
    {  
      if (digitalRead(ZoneInst) == LOW)   // Test si intrusion sur la zone instantanée ?
       { 
         MemZoneI = 1;                    // Mémorise alarme sur la zone instantannée
         Sirene() ;                       // Active la sirène
       }
      if (digitalRead(ZoneRet) == LOW)  Entree();   // S'il y a une intrusion sur la zone Retardée -> Active la teporisation d'entrée
    }   
    
//---------------------------------------------------- La centrale vient d'être désarmée --------   

  if (MemZoneI == 0 & MemZoneR ==0)       // Test s'il n'y a pas de mémoire alarme ? 
    {Arret ();}
     else
    {
      lcd.clear();                        // Efface écran et place le curseur en haut à gauche 
      lcd.print("Memoire alarme  ");      // Affichage le message de mémoire alarme
      if (MemZoneI == 1)                  // Test s'il y a une mémoire alarme sur la zone instantanée ?
       {
       lcd.setCursor(0, 1);               // Place le curseur sur la 2ème ligne
       for (byte i=0; i<5; i++)           // La valeur 5 correspond au nombre de lettres composant le nom de la zone instantanée
         {
         lcd.print(NomZoneI[i]);          // Affiche le nom de la zone instantanée à l'origine de l'alarme
         }
       }

      if (MemZoneR == 1)                  // Test s'il y a une mémoire alarme sur la zone retardée ?
       {
       lcd.setCursor(8, 1);               // Place le curseur sur la 2ème ligne au milieu de l'écran
       for (byte i=0; i<7; i++)           // La valeur 7 correspond au nombre de lettres composant le nom de la zone retardée
         {
         lcd.print(NomZoneR[i]);          // Affiche le nom de la zone reetardée à l'origine de l'alarme
         }
       }
    }
}

//--------------------------------------------- Fonctions -----------------------------------------------
 
    void Entree() 
    {  
      Tempo = TempoEntree;                 // Récupère la valeur de la temporisation d'entrée
      lcd.clear();                         // Efface l'écran et place le curseur en haut à gauche 
      lcd.print("Detection zone  ");       // Affichage du message
      lcd.setCursor(0, 1);                 // Place le curseur sur la 2ème ligne
      lcd.print("Tempo Entree    ");
      while (digitalRead(Clef) == LOW  &  Tempo !=255)   // Tant que l'on a pas mis la centrale d'alarme à l'arrêt et que la tempo n'est pas finie
      { 
        lcd.setCursor(13, 1);              // Place le curseur sur la 2ème ligne pour afficher la temporiation
        lcd.print(Tempo);                  // Affiche la valeur de la temporisation d'entrée
        lcd.print("  ");                   // Affiche un caractère à "vide" pour effacer le digit passé sous la valeur 9
        Tempo--;                           // Décompte le durée de la temporisation d'entrée
        delay(1000);                       // Temporisation d'une seconde
      }
      if (Tempo ==255)                     // Regarde si on est arrivé à la fin de la tempo d'entrée ?
       {
         MemZoneR = 1;                     // Si tel est le cas, on mémorise l'alarme et 
         Sirene() ;                        // on active la sirène
      }
  }
//---------------------------------------------
 
    void Sirene() 
    {  
      Tempo = TempoAlarme;                 // Récupère la valeur de la temporisation d'alarme
      lcd.clear();                         // Efface l'écran et place le curseur en haut à gauche 
      lcd.print("Alarme Intrusion");       // Affichage du message
      lcd.setCursor(0, 1);                 // Place le curseur sur la 2ème ligne
      lcd.print("Temp Alarme     ");
      while (digitalRead(Clef) == LOW  &  Tempo !=255)   // Tant que l'on a pas mis la centrale d'alarme à l'arrêt et que la tempo n'est pas finie
      { 
        lcd.setCursor(13, 1);              // Place le curseur sur la 2ème ligne pour afficher la temporiation
        lcd.print(Tempo);                  // Affiche la valeur de la temporisation d'alarme
        lcd.print("  ");                   // Affiche un caractère à "vide" pour effacer le digit passé sous la valeur 9
        Tempo--;                           // Decompte durée temporisation d'alarme
        for (byte i=5; i<100; i++) 
         {    
           analogWrite(Buzzer, i);         // Génère bruit de la sirène sur le buzzer
           delay(10);
         }
           delay(750);
           analogWrite(Buzzer, 0);
           delay(100);
      }
     Actif();                              // Affiche de nouveau que le systeme d'alarme est actif 
  }

//---------------------------------------------
 
  void Arret ()
  { 
     lcd.clear();                          // Efface l'écran et place le curseur en haut à gauche 
     lcd.print("Systeme d'alarme");        // Affichage de l'état de la centrale d'alarme
     lcd.setCursor(0, 1);                  // Place le curseur sur la 2ème ligne
     lcd.print("   a l'arret    ");  
   }

//---------------------------------------------
 
   void Actif()
  {
    lcd.clear();                            // Efface l'écran et place le curseur en haut à gauche 
    lcd.print("Systeme d'alarme");          // Affichage de l'état de la centrale d'alarme
    lcd.setCursor(0, 1);                    // Place le curseur sur la 2ème ligne
    lcd.print("     ACTIF      ");
  }  
   
