/* Utilisation du module MIKROE-1993
   Cette application va vous permettre de réaliser une petite station météo à l'aide
   de la matrice du module "7x10 R Click" (à mettre sur le support A de la platine "Flip & Click")
   et du module "DHT22 click" (à mettre sur le support B de la platine "Flip & Click")

   CE PROGRAMME EST FOURNI «TEL QUEL», SANS GARANTIE D'AUCUNE SORTE, EXPLICITE OU
   IMPLICITE, Y COMPRIS MAIS SANS S'Y LIMITER LES GARANTIES DE QUALITÉ MARCHANDE,
   D'ADAPTATION À UN USAGE PARTICULIER ET D'ABSENCE DE CONTREFAÇON. EN AUCUN CAS, LA
   RESPONSABILITE DES AUTEURS OU DES DÉTENTEURS DE DROITS D'AUTEUR NE SERONT TENUS 
   RESPONSABLES DE TOUTE RECLAMATION, DOMMAGE OU AUTRE RESPONSABILITÉ, QUE CE SOIT
   DANS UNE ACTION DE CONTRAT, TORT OU AUTREMENT, PROVENANT DE, OU EN RELATION AVEC
   CE PROGRAMME OU L'UTILISATION OU AUTRES TRAITEMENTS DU PROGRAMME. CE PROGRAMME 
   EST PUBLIE SOUS LICENCE MIT.
    
*/

#include "DHT.h"                                      // Integration de la librairie DHT
#include <SPI.h>                                      // Intégration de la librairie SPI

#define DHTPIN 87                                     // Attribution de la broche gérant le capteur DHT22
#define DHTTYPE DHT22                                 // Défini le type de capteur utilisé
#define CS0 77                                        // Attribution du signal CS0 au port 77
#define RCLK 54                                       // Attribution du signal RCLK au port 54
#define MR 33                                         // Attribution du signal MR au port 33
#define RST 6                                         // Attribution du signal RST au port 6

DHT dht(DHTPIN, DHTTYPE);                             // Initialisation pour pilotage du capteur
byte Pos;                                             // Variable gestion de la position dans le message 
byte Decal;                                           // Variable calcule position dans la matrice
byte Espace;                                          // Variable pour gérer un décalable moindre en cas d'espace
byte Dizaine,Unite;                                   // Variable pour gestion de l'affichage de la température et de l'humidité
const byte Vitesse = 4;                               // Vitesse de déplacement du scrolling
 
const byte Soleil[6][7] ={{0b00000000,0b00000000,0b00000001,0b00000101,0b00000001,0b00000000,0b00000000},    // Animation Soleil  
                          {0b00010000,0b00000000,0b00011000,0b00001010,0b00011000,0b00000000,0b00010000}, 
                          {0b00000000,0b00000010,0b00000001,0b00000101,0b00000001,0b00000010,0b00000000}, 
                          {0b00010000,0b00000100,0b00011000,0b00001010,0b00011000,0b00000100,0b00010000}, 
                          {0b00000000,0b00000010,0b00000001,0b00000111,0b00000001,0b00000010,0b00000000}, 
                          {0b00010000,0b00010100,0b00011000,0b00011110,0b00011000,0b00010100,0b00010000}};                                         

const byte Nuage[8][7]  ={{0b00000000,0b00000011,0b00000111,0b00000111,0b00000011,0b00000000,0b00000000},    // Animation Nuage  
                          {0b00000000,0b00011110,0b00011111,0b00011111,0b00011110,0b00000000,0b00000000}, 
                          {0b00000000,0b00000111,0b00001111,0b00001111,0b00000111,0b00000000,0b00000000}, 
                          {0b00000000,0b00011100,0b00011110,0b00011110,0b00011100,0b00000000,0b00000000}, 
                          {0b00000000,0b00001111,0b00011111,0b00011111,0b00001111,0b00000000,0b00000000}, 
                          {0b00000000,0b00011000,0b00011100,0b00011100,0b00011000,0b00000000,0b00000000},
                          {0b00000000,0b00000111,0b00001111,0b00001111,0b00000111,0b00000000,0b00000000}, 
                          {0b00000000,0b00011100,0b00011110,0b00011110,0b00011100,0b00000000,0b000000000}};  
                          
const byte Pluie[10][7] ={{0b00000000,0b00000000,0b00000100,0b00000100,0b00001110,0b00000100,0b00000000},    // Animation Pluie  
                          {0b00000100,0b00000100,0b00001110,0b00000100,0b00000000,0b00000000,0b00000000}, 
                          {0b00000000,0b00000000,0b00000000,0b00000000,0b00000100,0b00000100,0b00001110}, 
                          {0b00000000,0b00000000,0b00000100,0b00000100,0b00001110,0b00000100,0b00000000}, 
                          {0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000100}, 
                          {0b00000000,0b00000000,0b00000000,0b00000000,0b00000100,0b00000100,0b00001110},  
                          {0b00001110,0b00000100,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000}, 
                          {0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000100},
                          {0b00000100,0b00000100,0b00001110,0b00000100,0b00000000,0b00000000,0b00000000}, 
                          {0b00001110,0b00000100,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000}};

char Message[] = "              ";      // Contenu du message défilant
                            
const byte MLettres[40][7] ={{0b01110000,0b10001000,0b10001000,0b11111000,0b10001000,0b10001000,0b10001000},   // Lettre A  
                            {0b11110000,0b10001000,0b10001000,0b11110000,0b10001000,0b10001000,0b11110000},    // Lettre B
                            {0b01111000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b01111000},    // Lettre C
                            {0b11110000,0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b11110000},    // Lettre D  
                            {0b11111000,0b10000000,0b10000000,0b11110000,0b10000000,0b10000000,0b11111000},    // Lettre E
                            {0b11111000,0b10000000,0b10000000,0b11110000,0b10000000,0b10000000,0b10000000},    // Lettre F
                            {0b11111000,0b10000000,0b10000000,0b10111000,0b10001000,0b10001000,0b11111000},    // Lettre G
                            {0b10001000,0b10001000,0b10001000,0b11111000,0b10001000,0b10001000,0b10001000},    // Lettre H
                            {0b11111000,0b00100000,0b00100000,0b00100000,0b00100000,0b00100000,0b11111000},    // Lettre I                                                                                   {0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000},    // Lettre H
                            {0b11111000,0b00100000,0b00100000,0b00100000,0b00100000,0b00100000,0b11000000},    // Lettre J
                            {0b10001000,0b10010000,0b10100000,0b11000000,0b10100000,0b10010000,0b10001000},    // Lettre K
                            {0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b11111000},    // Lettre L
                            {0b10001000,0b11011000,0b10101000,0b10001000,0b10001000,0b10001000,0b10001000},    // Lettre M
                            {0b10001000,0b10001000,0b11001000,0b10101000,0b10011000,0b10001000,0b10001000},    // Lettre N
                            {0b01110000,0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b01110000},    // Lettre O
                            {0b11110000,0b10001000,0b10001000,0b11110000,0b10000000,0b10000000,0b10000000},    // Lettre P
                            {0b01110000,0b10001000,0b10001000,0b10001000,0b10101000,0b10010000,0b01101000},    // Lettre Q
                            {0b11110000,0b10001000,0b10001000,0b11110000,0b10010000,0b10001000,0b10001000},    // Lettre R
                            {0b01110000,0b10001000,0b10000000,0b01110000,0b00001000,0b10001000,0b01110000},    // Lettre S
                            {0b11111000,0b00100000,0b00100000,0b00100000,0b00100000,0b00100000,0b00100000},    // Lettre T
                            {0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b01110000},    // Lettre U
                            {0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b01010000,0b00100000},    // Lettre V
                            {0b10001000,0b10001000,0b10101000,0b10101000,0b11011000,0b10001000,0b10001000},    // Lettre W
                            {0b10001000,0b10001000,0b01010000,0b00100000,0b01010000,0b10001000,0b10001000},    // Lettre X
                            {0b10001000,0b10001000,0b01010000,0b00100000,0b00100000,0b00100000,0b00100000},    // Lettre Y
                            {0b11111000,0b00001000,0b00010000,0b00100000,0b01000000,0b10000000,0b11111000},    // Lettre Z
                            {0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000},    // ESPACE 
                            {0b11111000,0b10001000,0b10001000,0b10001000,0b10001000,0b10001000,0b11111000},    // Chiffre 0                                                                                                                                                                                                                                                        
                            {0b00100000,0b01100000,0b10100000,0b00100000,0b00100000,0b00100000,0b11111000},    // Chiffre 1 
                            {0b11111000,0b00001000,0b00001000,0b11111000,0b10000000,0b10000000,0b11111000},    // Chiffre 2 
                            {0b11111000,0b00001000,0b00001000,0b01111000,0b00001000,0b00001000,0b11111000},    // Chiffre 3 
                            {0b00010000,0b00110000,0b01010000,0b11111000,0b00010000,0b00010000,0b00010000},    // Chiffre 4 
                            {0b11111000,0b10000000,0b10000000,0b11111000,0b00001000,0b00001000,0b11111000},    // Chiffre 5                             
                            {0b11111000,0b10000000,0b10000000,0b11111000,0b10001000,0b10001000,0b11111000},    // Chiffre 6 
                            {0b11111000,0b00001000,0b00010000,0b00100000,0b01000000,0b01000000,0b01000000},    // Chiffre 7 
                            {0b11111000,0b10001000,0b10001000,0b11111000,0b10001000,0b10001000,0b11111000},    // Chiffre 8 
                            {0b11111000,0b10001000,0b10001000,0b11111000,0b00001000,0b00001000,0b11111000},    // Chiffre 9          
                            {0b10011000,0b00100000,0b00100000,0b00011000,0b00000000,0b00000000,0b00000000},    // °C 
                            {0b00000000,0b11001000,0b11010000,0b00100000,0b01011000,0b10011000,0b00000000},    // % 
                            {0b00000000,0b00000000,0b00000000,0b01110000,0b00000000,0b00000000,0b00000000}};   // -                            
                           
byte Matrice[2][7];                                   // Définition de la matrice (aire de jeu)
byte k;                                               // Variable d'usage générale
byte Temp,Humid;                                      // Variables pour stockage température et humidité
                               
void setup() {
  pinMode(CS0, OUTPUT);pinMode(RCLK, OUTPUT);         // Les broches CS0 et RCLK sont des sorties
  pinMode(MR, OUTPUT);pinMode(RST, OUTPUT);           // Les broches MR et RST sont des sorties
  digitalWrite(MR, HIGH);                             // Force la broche MR au niveau logique haut
  digitalWrite(RCLK, LOW);                            // Niveau logique bas sur l'horloge du 4017 du module 7x10 R click
  digitalWrite(RST, LOW);                             // Niveau logique bas sur l'entrée de RESET du 4017 du module 7x10 R click
  SPI.begin();                                        // Initialiation du port SPI
  dht.begin();                                        // Initialisation communication avec le capteur DHT22
  delay (3000);                                       // Temporisation initialisation
} 

void loop() { 
  Humid = dht.readHumidity();                         // Lecture de l'humidité mesurée par le capteur
  Temp = dht.readTemperature();                       // Lecture de la température mesurée par le capteur
  if (Humid==0 & Temp==0) 
  {Message[0]=69;Message[1]=82;Message[2]=82;Message[3]=69;Message[4]=85;Message[5]=82;Message[6]=91;Message[7]=68;Message[8]=72;Message[9]=84;Message[10]=50;
  Message[11]=50;Message[12]=91;Message[13]=91;;Message[14]=91;Message[15]=42;}    // Message problème avec capteur si Temp et Hum = 0;
  else
  {Message[0]=77;Message[1]=69;Message[2]=84;Message[3]=69;Message[4]=79;Message[5]=91;Message[6]=91;Message[7]=91;Message[8]=91;Message[9]=42;}    // Message "METEO"
   
  Pos = 0;                                            // On se place sur le premier caractère du message "METEO ou ERR"
  do
  {Scrolling ();Pos++;
  } while (Message[Pos]!=42);                         // Tant que l'on est pas arrivé à la fin du message

  if (Humid!=0 & Temp!=0) {                           // Si on reçoit bien des données affiche la température et l'humidité
    if (Humid >= 59) APluie();                        // Si taux d'humidité élevé... annonce la pluie   
    if (Humid > 49 & Humid < 59) ANuage();            // Si taux d'humidité moyen... annonce les nuages
    if (Humid <=49) ASoleil();                        // Si taux d'humidité bas... annonce le soleil

    
    Dizaine = 0;                                                                                                // Récupération des dizaines et centaines (Temperature) 
    while (Temp > 9)
    {Temp=Temp-10;Dizaine++;}
    Unite = Temp;  
    
    Message[0]=91;Message[1]=91;Message[2]=91;Message[3]=48+Dizaine;Message[4]=48+Unite;Message[5]=58;Message[6]=60;  // Composition du message (ajoute température)

    Dizaine = 0;                                                                                                // Récupération des dizaines et centaines (Humidité) 
    while (Humid > 9)
    {Humid=Humid-10;Dizaine++;}
    Unite = Humid;  

    Message[7]=48+Dizaine;Message[8]=48+Unite;Message[9]=59;Message[10]=91;Message[11]=91;;Message[12]=91;;Message[13]=91;;Message[14]=91;Message[15]=42;

    Pos = 0;                                                                                                      // On se place sur le premier caractère du message
    do
    {Scrolling ();Pos++;
    } while (Message[Pos]!=42);}}                                                                                 // Tant que l'on est pas arrivé à la fin du message
                                                                                                               

void APluie() {k=0;                                                                                             // Animation Pluie
    for (byte j = 0 ; j < 44; j++){
       for (byte i = 0 ; i < 7; i++){Matrice[0][i]=Pluie[k][i];Matrice[1][i]=Pluie[k+1][i];}                    // Transfert images de la puie dans la matrice
       for (byte i = 0 ; i < 7; i++){Rmatrice();} k=k+2; if (k==10) k=0;}}                                      // Temporisation entre les images

void ANuage() {k=0;                                                                                             // Animation Nuage
    for (byte j = 0 ; j < 18; j++){ 
       for (byte i = 0 ; i < 7; i++){Matrice[0][i]=Nuage[k][i];Matrice[1][i]=Nuage[k+1][i];}                    // Transfert images des nuages dans la matrice
       for (byte i = 0 ; i < 13; i++){Rmatrice();} k=k+2; if (k==8) k=0;}}                                      // Temporisation entre les images

void ASoleil() {k=0;                                                                                            // Animation soleil
    for (byte j = 0 ; j < 22; j++){
       for (byte i = 0 ; i < 7; i++){Matrice[0][i]=Soleil[k][i];Matrice[1][i]=Soleil[k+1][i];}                  // Transfert images du soleil dans la matrice
       for (byte i = 0 ; i < 10; i++){Rmatrice();} k=k+2; if (k==6) k=0;}}                                      // Temporisation entre les images

void Scrolling (){                                                                                              // Fonction permettant de réaliser le scrolling
    if (Message[Pos]>60) {Decal=65;}else{Decal=21;}                                                             // Repositionnement dans la matrice chiffre/lettre 
    Espace = 6;if (Message[Pos]==91) Espace = 3;
    for (byte j = 0 ; j < Espace; j++)
      {for (byte k = 0 ; k < Vitesse; k++) {Rmatrice();}                                                        // Temporisation gérant la vitesse du scrolling
       for (byte i = 0 ; i < 7; i++){Matrice[1][i]=Matrice[1][i]>>1;
       bitWrite(Matrice[1][i],4,bitRead(Matrice[0][i],0));   
       Matrice[0][i]=Matrice[0][i]>>1;   bitWrite(Matrice[0][i],4,bitRead(MLettres[Message[Pos]-Decal][i],7-j));}}}

void Rmatrice() {                                                                                                // Fonction permettant de rafraichir la matrice
     digitalWrite(RST, HIGH);digitalWrite(RST, LOW);                                                             // Reset du 4017 du module 7x10 R click
     for (byte i = 0 ; i < 7; i++){                                                                              // Boucle générant 7 coups d'horloge sur le 4017 
     digitalWrite(CS0, LOW);                                                                                     // Sélection broche CSO    
     SPI.transfer(Matrice[0][i]); SPI.transfer(Matrice[1][i]);                                                   // Envoi les données aux matrices
     digitalWrite(CS0, HIGH);delay(3);                                                                           // Dé-sélection de la broche CS0 
     digitalWrite(MR, LOW);digitalWrite(MR, HIGH);                                                               // Vide le registre à décalage des 74HC595          
     digitalWrite(RCLK, HIGH);digitalWrite(RCLK, LOW);}                                                          // Coup d'horloge sur le 4017 du module 7x10 R click
     digitalWrite(CS0, LOW);SPI.transfer(Matrice[0][0]); SPI.transfer(Matrice[1][0]);digitalWrite(CS0, HIGH);}
 
